#lang racket

(require vigracket)

(define imgdir (current-directory))
(define img (loadimage (build-path imgdir 
                                "cells.jpg")))
(show-image img)

(define mask (image-map (lambda (x) (if (< x 220.0)
                                        255.0
                                        0.0))
                        (image->red img)))
(show-image mask "Raw mask image")

(define omask (openingimage mask 1))
(show-image omask "Opened mask image")

(define labels
           (labelimage omask #t 0.0))

(define stats (extractfeatures img labels))

(define (circle-like? region threshold)
    (let* [(width (- (image-ref stats 3 region 0)
                     (image-ref stats 1 region 0)))
           (height (- (image-ref stats 4 region 0)
                      (image-ref stats 2 region 0)))
           (a (max width height))
           (b (min width height))]
       (< (/ a b) threshold)))


(define maxlabel (inexact->exact (car (image-reduce max labels 0.0))))
(define label-ids (build-list (+ maxlabel 1) values))
(define filtered-ids (filter (curryr circle-like? 4/3) label-ids))

(define circle-likes
 (image-map (lambda (v) (if (and (> v 0.0)
                                 (list? (memv (inexact->exact v) filtered-ids)))
                            255.0
                            0.0))
            labels))

(define non-circle-likes
 (image-map (lambda (v) (if (and (> v 0.0)
                                 (boolean? (memv (inexact->exact v) filtered-ids)))
                            255.0
                            0.0))
            labels))
(define empty (make-image (image-width omask) (image-height omask) 1 0.0))

(define result (list (car non-circle-likes) (car circle-likes) (car empty)))
(show-image result)
(saveimage result (build-path imgdir "result.png"))
